@@ -18,7 +18,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -35,7 +35,7 @@
 ;;   simply saved as global variables.
 
 ;; References:
-;; + ICCCM (http://www.x.org/releases/X11R7.7/doc/xorg-docs/icccm/icccm.txt)
+;; + ICCCM (https://www.x.org/releases/X11R7.7/doc/xorg-docs/icccm/icccm.txt)
 ;; + xcb/util-wm (git://anongit.freedesktop.org/xcb/util-wm)
 
 ;;; Code:
@@ -47,9 +47,12 @@
 
 (eval-and-compile
   (defconst xcb:icccm:-atoms
-    '(UTF8_STRING COMPOUND_TEXT TEXT C_STRING MANAGER
-                  WM_PROTOCOLS WM_TAKE_FOCUS WM_DELETE_WINDOW
-                  WM_STATE WM_CHANGE_STATE WM_S0)
+    '(;; Standard atoms
+      UTF8_STRING COMPOUND_TEXT TEXT C_STRING MANAGER
+      WM_PROTOCOLS WM_TAKE_FOCUS WM_DELETE_WINDOW
+      WM_STATE WM_CHANGE_STATE WM_S0
+      ;; Illegal alias of UTF8_STRING used by Valve's Steam.
+      UTF-8)
     "Atoms involved in ICCCM.")
 
   (dolist (atom xcb:icccm:-atoms)
@@ -140,39 +143,33 @@ A valid timestamp (rather than `xcb:Time:CurrentTime') must be supplied.")
 according to BYTE-ARRAY.
 
 This method automatically format the value as 8, 16 or 32 bits array."
-  (let ((retval (cl-call-next-method obj byte-array))
-        tmp)
+  (let ((retval (cl-call-next-method obj byte-array)))
     (with-slots (~lsb length format value-len value) obj
       (if (or (= 0 value-len) (= 0 length))
           (setf value nil)              ;no available value
-        (setq tmp value                 ;long-offset is always 0
-              value nil)
-        (pcase format
-          (8
-           (cl-assert (= value-len (length tmp)))
-           (setf value tmp))
-          (16
-           (cl-assert (= (* 2 value-len) (length tmp)))
-           (if ~lsb
-               (dotimes (_ value-len)
-                 (setf value (vconcat value
-                                      (vector (xcb:-unpack-u2-lsb tmp 0))))
-                 (setq tmp (substring tmp 2)))
-             (dotimes (_ value-len)
-               (setf value (vconcat value
-                                    (vector (xcb:-unpack-u2 tmp 0))))
-               (setq tmp (substring tmp 2)))))
-          (32
-           (cl-assert (= (* 4 value-len) (length tmp)))
-           (if ~lsb
-               (dotimes (_ value-len)
-                 (setf value (vconcat value
-                                      (vector (xcb:-unpack-u4-lsb tmp 0))))
-                 (setq tmp (substring tmp 4)))
-             (dotimes (_ value-len)
-               (setf value (vconcat value (vector (xcb:-unpack-u4 tmp 0))))
-               (setq tmp (substring tmp 4)))))
-          (_ (cl-assert nil)))))
+        (let ((tmp value))              ;long-offset is always 0
+          (pcase format
+            (8
+             (cl-assert (= value-len (length tmp))))
+            (16
+             (cl-assert (= (* 2 value-len) (length tmp)))
+             (let ((newval (make-vector value-len 0)))
+               (if ~lsb
+                   (dotimes (idx value-len)
+                     (aset newval idx (xcb:-unpack-u2-lsb tmp (* 2 idx))))
+                 (dotimes (idx value-len)
+                   (aset newval idx (xcb:-unpack-u2 tmp (* 2 idx)))))
+               (setf value newval)))
+            (32
+             (cl-assert (= (* 4 value-len) (length tmp)))
+             (let ((newval (make-vector value-len 0)))
+               (if ~lsb
+                   (dotimes (idx value-len)
+                     (aset newval idx (xcb:-unpack-u4-lsb tmp (* 4 idx))))
+                 (dotimes (idx value-len)
+                   (aset newval idx (xcb:-unpack-u4 tmp (* 4 idx)))))
+               (setf value newval)))
+            (_ (cl-assert nil))))))
     retval))
 
 (defclass xcb:icccm:-ChangeProperty (xcb:ChangeProperty)
@@ -222,14 +219,17 @@ This method automatically decodes the value (as string)."
         (setf value
               (decode-coding-string
                (apply #'unibyte-string (append value nil))
-               (cond ((= type xcb:Atom:UTF8_STRING) 'utf-8)
+               (cond ((or (= type xcb:Atom:UTF8_STRING)
+                          (= type xcb:Atom:UTF-8))
+                      'utf-8)
                      ((= type xcb:Atom:STRING) 'iso-latin-1)
                      ((= type xcb:Atom:COMPOUND_TEXT)
                       'compound-text-with-extensions)
-                     ((or (eq type xcb:Atom:TEXT) (eq type xcb:Atom:C_STRING))
+                     ((or (= type xcb:Atom:TEXT)
+                          (= type xcb:Atom:C_STRING))
                       'no-conversion)
-                     (t (error "[XELB:ICCCM] Unsupported encoding: %d"
-                               type)))))))
+                     (t (error "[XELB:ICCCM] Unsupported encoding: %s (%d)"
+                               (x-get-atom-name type) type)))))))
     retval))
 
 (defclass xcb:icccm:-ChangeProperty-text (xcb:icccm:-ChangeProperty)
@@ -254,7 +254,8 @@ This method automatically encodes the data (which is a string)."
                    'compound-text-with-extensions)
                   ((or (eq type xcb:Atom:TEXT) (eq type xcb:Atom:C_STRING))
                    'no-conversion)
-                  (t (error "[XELB:ICCCM] Unsupported encoding: %d" type)))))))
+                  (t (error "[XELB:ICCCM] Unsupported encoding: %s (%d)"
+                            (x-get-atom-name type) type)))))))
   (cl-call-next-method obj))
 
 ;;;; Abstract classes for getting/changing single field properties
