@@ -17,7 +17,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -44,7 +44,7 @@
 ;; + <paramref> for `xcb:-marshal-field'?
 
 ;; References:
-;; + X protocol (http://www.x.org/releases/X11R7.7/doc/xproto/x11protocol.txt)
+;; + X protocol (https://www.x.org/releases/X11R7.7/doc/xproto/x11protocol.txt)
 
 ;;; Code:
 
@@ -916,6 +916,13 @@ The optional argument CTX is for <paramref>."
         (setf (slot-value obj (eieio-slot-descriptor-name slot)) (car tmp))))
     (slot-value obj '~size)))
 
+(defun xcb:unmarshal-new (type byte-array)
+  "Construct a new object of type TYPE and unmarshal BYTE-ARRAY into it.
+Return the new object."
+  (let ((obj (make-instance type)))
+    (xcb:unmarshal obj byte-array)
+    obj))
+
 
 
 (provide 'xcb-types)
